package com.sun.media.sound;

import java.util.Arrays;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.spi.FormatConversionProvider;

public final class AudioFloatFormatConverter extends FormatConversionProvider
{
    private final AudioFormat.Encoding[] formats;
    
    public AudioFloatFormatConverter() {
        this.formats = new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED, AudioFormat.Encoding.PCM_FLOAT };
    }
    
    @Override
    public AudioInputStream getAudioInputStream(AudioFormat.Encoding encoding, AudioInputStream audioInputStream) {
        if (audioInputStream.getFormat().getEncoding().equals(encoding)) {
            return audioInputStream;
        }
        AudioFormat format = audioInputStream.getFormat();
        int channels = format.getChannels();
        float sampleRate = format.getSampleRate();
        int sampleSizeInBits = format.getSampleSizeInBits();
        int bigEndian = format.isBigEndian();
        if (encoding.equals(AudioFormat.Encoding.PCM_FLOAT)) {
            sampleSizeInBits = 32;
        }
        return this.getAudioInputStream(new AudioFormat(encoding, sampleRate, sampleSizeInBits, channels, channels * sampleSizeInBits / 8, sampleRate, (boolean)bigEndian), audioInputStream);
    }
    
    @Override
    public AudioInputStream getAudioInputStream(AudioFormat audioFormat, AudioInputStream audioInputStream) {
        if (!this.isConversionSupported(audioFormat, audioInputStream.getFormat())) {
            throw new IllegalArgumentException("Unsupported conversion: " + audioInputStream.getFormat().toString() + " to " + audioFormat.toString());
        }
        return this.getAudioInputStream(audioFormat, AudioFloatInputStream.getInputStream(audioInputStream));
    }
    
    public AudioInputStream getAudioInputStream(AudioFormat format, AudioFloatInputStream audioFloatInputStream) {
        if (!this.isConversionSupported(format, audioFloatInputStream.getFormat())) {
            throw new IllegalArgumentException("Unsupported conversion: " + audioFloatInputStream.getFormat().toString() + " to " + format.toString());
        }
        if (format.getChannels() != audioFloatInputStream.getFormat().getChannels()) {
            audioFloatInputStream = new AudioFloatFormatConverter.AudioFloatInputStreamChannelMixer(audioFloatInputStream, format.getChannels());
        }
        if ((double)Math.abs(format.getSampleRate() - audioFloatInputStream.getFormat().getSampleRate()) > 1.0E-6) {
            audioFloatInputStream = new AudioFloatFormatConverter.AudioFloatInputStreamResampler(audioFloatInputStream, format);
        }
        return new AudioInputStream(new AudioFloatFormatConverter.AudioFloatFormatConverterInputStream(format, audioFloatInputStream), format, audioFloatInputStream.getFrameLength());
    }
    
    @Override
    public AudioFormat.Encoding[] getSourceEncodings() {
        return new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED, AudioFormat.Encoding.PCM_FLOAT };
    }
    
    @Override
    public AudioFormat.Encoding[] getTargetEncodings() {
        return new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED, AudioFormat.Encoding.PCM_FLOAT };
    }
    
    @Override
    public AudioFormat.Encoding[] getTargetEncodings(AudioFormat audioFormat) {
        if (AudioFloatConverter.getConverter(audioFormat) == null) {
            return new AudioFormat.Encoding[0];
        }
        return new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED, AudioFormat.Encoding.PCM_FLOAT };
    }
    
    @Override
    public AudioFormat[] getTargetFormats(AudioFormat.Encoding encoding, AudioFormat audioFormat) {
        if (AudioFloatConverter.getConverter(audioFormat) == null) {
            return new AudioFormat[0];
        }
        int channels = audioFormat.getChannels();
        ArrayList<AudioFormat> list = new ArrayList();
        if (encoding.equals(AudioFormat.Encoding.PCM_SIGNED)) {
            list.add(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, -1.0f, 8, channels, channels, -1.0f, false));
        }
        if (encoding.equals(AudioFormat.Encoding.PCM_UNSIGNED)) {
            list.add(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, -1.0f, 8, channels, channels, -1.0f, false));
        }
        for (int i = 16; i < 32; i += 8) {
            if (encoding.equals(AudioFormat.Encoding.PCM_SIGNED)) {
                list.add(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, -1.0f, i, channels, channels * i / 8, -1.0f, false));
                list.add(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, -1.0f, i, channels, channels * i / 8, -1.0f, true));
            }
            if (encoding.equals(AudioFormat.Encoding.PCM_UNSIGNED)) {
                list.add(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, -1.0f, i, channels, channels * i / 8, -1.0f, true));
                list.add(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, -1.0f, i, channels, channels * i / 8, -1.0f, false));
            }
        }
        if (encoding.equals(AudioFormat.Encoding.PCM_FLOAT)) {
            list.add(new AudioFormat(AudioFormat.Encoding.PCM_FLOAT, -1.0f, 32, channels, channels * 4, -1.0f, false));
            list.add(new AudioFormat(AudioFormat.Encoding.PCM_FLOAT, -1.0f, 32, channels, channels * 4, -1.0f, true));
            list.add(new AudioFormat(AudioFormat.Encoding.PCM_FLOAT, -1.0f, 64, channels, channels * 8, -1.0f, false));
            list.add(new AudioFormat(AudioFormat.Encoding.PCM_FLOAT, -1.0f, 64, channels, channels * 8, -1.0f, true));
        }
        return (AudioFormat[])list.toArray(new AudioFormat[list.size()]);
    }
    
    @Override
    public boolean isConversionSupported(AudioFormat audioFormat, AudioFormat audioFormat2) {
        return AudioFloatConverter.getConverter(audioFormat2) != null && AudioFloatConverter.getConverter(audioFormat) != null && audioFormat2.getChannels() > 0 && audioFormat.getChannels() > 0;
    }
    
    @Override
    public boolean isConversionSupported(AudioFormat.Encoding encoding, AudioFormat audioFormat) {
        if (AudioFloatConverter.getConverter(audioFormat) == null) {
            return false;
        }
        int i = 0;
        while (i < this.formats.length) {
            if (encoding.equals(this.formats[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private static class AudioFloatFormatConverterInputStream extends InputStream
    {
        private final AudioFloatConverter converter;
        private final AudioFloatInputStream stream;
        private float[] readfloatbuffer;
        private final int fsize;
        
        AudioFloatFormatConverterInputStream(AudioFormat audioFormat, AudioFloatInputStream stream) {
            this.stream = stream;
            this.converter = AudioFloatConverter.getConverter(audioFormat);
            this.fsize = (audioFormat.getSampleSizeInBits() + 7) / 8;
        }
        
        @Override
        public int read() throws IOException {
            byte[] b = { 0 };
            int read = super.read(b);
            if (read < 0) {
                return read;
            }
            return b[0] & 0xFF;
        }
        
        @Override
        public int read(byte[] array, int n, int n2) throws IOException {
            int n3 = n2 / this.fsize;
            if (this.readfloatbuffer == null || this.readfloatbuffer.length < n3) {
                this.readfloatbuffer = new float[n3];
            }
            int read = this.stream.read(this.readfloatbuffer, 0, n3);
            if (read < 0) {
                return read;
            }
            this.converter.toByteArray(this.readfloatbuffer, 0, read, array, n);
            return read * this.fsize;
        }
        
        @Override
        public int available() throws IOException {
            int available = this.stream.available();
            if (available < 0) {
                return available;
            }
            return available * this.fsize;
        }
        
        @Override
        public void close() throws IOException {
            this.stream.close();
        }
        
        @Override
        public synchronized void mark(int n) {
            this.stream.mark(n * this.fsize);
        }
        
        @Override
        public boolean markSupported() {
            return this.stream.markSupported();
        }
        
        @Override
        public synchronized void reset() throws IOException {
            this.stream.reset();
        }
        
        @Override
        public long skip(long n) throws IOException {
            long skip = this.stream.skip(n / (long)this.fsize);
            if (skip < 0L) {
                return skip;
            }
            return skip * (long)this.fsize;
        }
    }
    
    private static class AudioFloatInputStreamChannelMixer extends AudioFloatInputStream
    {
        private final int targetChannels;
        private final int sourceChannels;
        private final AudioFloatInputStream ais;
        private final AudioFormat targetFormat;
        private float[] conversion_buffer;
        
        AudioFloatInputStreamChannelMixer(AudioFloatInputStream ais, int n) {
            this.sourceChannels = ais.getFormat().getChannels();
            this.targetChannels = n;
            this.ais = ais;
            AudioFormat format = ais.getFormat();
            this.targetFormat = new AudioFormat(format.getEncoding(), format.getSampleRate(), format.getSampleSizeInBits(), n, format.getFrameSize() / this.sourceChannels * n, format.getFrameRate(), format.isBigEndian());
        }
        
        @Override
        public int available() throws IOException {
            return this.ais.available() / this.sourceChannels * this.targetChannels;
        }
        
        @Override
        public void close() throws IOException {
            this.ais.close();
        }
        
        @Override
        public AudioFormat getFormat() {
            return this.targetFormat;
        }
        
        @Override
        public long getFrameLength() {
            return this.ais.getFrameLength();
        }
        
        @Override
        public void mark(int n) {
            this.ais.mark(n / this.targetChannels * this.sourceChannels);
        }
        
        @Override
        public boolean markSupported() {
            return this.ais.markSupported();
        }
        
        @Override
        public int read(float[] array, int n, int n2) throws IOException {
            int n3 = n2 / this.targetChannels * this.sourceChannels;
            if (this.conversion_buffer == null || this.conversion_buffer.length < n3) {
                this.conversion_buffer = new float[n3];
            }
            int read = this.ais.read(this.conversion_buffer, 0, n3);
            if (read < 0) {
                return read;
            }
            if (this.sourceChannels == 1) {
                int targetChannels2 = this.targetChannels;
                for (int n12 = 0; n12 < this.targetChannels; ++n12) {
                    for (int n13 = 0, n14 = n + n12; n13 < n3; ++n13, n14 += targetChannels2) {
                        array[n14] = this.conversion_buffer[n13];
                    }
                }
            }
            else if (this.targetChannels == 1) {
                int sourceChannels2 = this.sourceChannels;
                for (int n5 = 0, n6 = n; n5 < n3; n5 += sourceChannels2, ++n6) {
                    array[n6] = this.conversion_buffer[n5];
                }
                for (int n7 = 1; n7 < this.sourceChannels; ++n7) {
                    int n8;
                    for (, ; n7 < n3; n7 += sourceChannels2, ++n) {
                        n8 = n;
                        array[n8] += this.conversion_buffer[n7];
                    }
                }
                float n9 = 1.0f / (float)this.sourceChannels;
                int n10;
                int n11;
                for (n10 = 0, ; n10 < n3; n10 += sourceChannels2, ++n) {
                    n11 = n;
                    array[n11] *= n9;
                }
            }
            else {
                int i = Math.min(this.sourceChannels, this.targetChannels);
                int n4 = n + n2;
                int targetChannels = this.targetChannels;
                int sourceChannels = this.sourceChannels;
                for (int j = 0; j < i; ++j) {
                    for (int k = n + j, j = j; k < n4; k += targetChannels, j += sourceChannels) {
                        array[k] = this.conversion_buffer[j];
                    }
                }
                for (; i < this.targetChannels; ++i) {
                    for (int l = n + i; l < n4; l += targetChannels) {
                        array[l] = 0.0f;
                    }
                }
            }
            return read / this.sourceChannels * this.targetChannels;
        }
        
        @Override
        public void reset() throws IOException {
            this.ais.reset();
        }
        
        @Override
        public long skip(long n) throws IOException {
            long skip = this.ais.skip(n / (long)this.targetChannels * (long)this.sourceChannels);
            if (skip < 0L) {
                return skip;
            }
            return skip / (long)this.sourceChannels * (long)this.targetChannels;
        }
    }
    
    private static class AudioFloatInputStreamResampler extends AudioFloatInputStream
    {
        private final AudioFloatInputStream ais;
        private final AudioFormat targetFormat;
        private float[] skipbuffer;
        private SoftAbstractResampler resampler;
        private final float[] pitch;
        private final float[] ibuffer2;
        private final float[][] ibuffer;
        private float ibuffer_index;
        private int ibuffer_len;
        private final int nrofchannels;
        private float[][] cbuffer;
        private final int buffer_len = 512;
        private final int pad;
        private final int pad2;
        private final float[] ix;
        private final int[] ox;
        private float[][] mark_ibuffer;
        private float mark_ibuffer_index;
        private int mark_ibuffer_len;
        
        AudioFloatInputStreamResampler(AudioFloatInputStream ais, AudioFormat audioFormat) {
            this.pitch = new float[1];
            this.ibuffer_index = 0.0f;
            this.ibuffer_len = 0;
            this.ix = new float[1];
            this.ox = new int[1];
            this.mark_ibuffer = (float[][])null;
            this.mark_ibuffer_index = 0.0f;
            this.mark_ibuffer_len = 0;
            this.ais = ais;
            AudioFormat format = ais.getFormat();
            this.targetFormat = new AudioFormat(format.getEncoding(), audioFormat.getSampleRate(), format.getSampleSizeInBits(), format.getChannels(), format.getFrameSize(), audioFormat.getSampleRate(), format.isBigEndian());
            this.nrofchannels = this.targetFormat.getChannels();
            Object property = audioFormat.getProperty("interpolation");
            if (property != null && property instanceof String) {
                String s = (String)property;
                if (s.equalsIgnoreCase("point")) {
                    this.resampler = new SoftPointResampler();
                }
                if (s.equalsIgnoreCase("linear")) {
                    this.resampler = new SoftLinearResampler2();
                }
                if (s.equalsIgnoreCase("linear1")) {
                    this.resampler = new SoftLinearResampler();
                }
                if (s.equalsIgnoreCase("linear2")) {
                    this.resampler = new SoftLinearResampler2();
                }
                if (s.equalsIgnoreCase("cubic")) {
                    this.resampler = new SoftCubicResampler();
                }
                if (s.equalsIgnoreCase("lanczos")) {
                    this.resampler = new SoftLanczosResampler();
                }
                if (s.equalsIgnoreCase("sinc")) {
                    this.resampler = new SoftSincResampler();
                }
            }
            if (this.resampler == null) {
                this.resampler = new SoftLinearResampler2();
            }
            this.pitch[0] = format.getSampleRate() / audioFormat.getSampleRate();
            this.pad = this.resampler.getPadding();
            this.pad2 = this.pad * 2;
            this.ibuffer = new float[this.nrofchannels][512 + this.pad2];
            this.ibuffer2 = new float[this.nrofchannels * 512];
            this.ibuffer_index = (float)(512 + this.pad);
            this.ibuffer_len = 512;
        }
        
        @Override
        public int available() throws IOException {
            return 0;
        }
        
        @Override
        public void close() throws IOException {
            this.ais.close();
        }
        
        @Override
        public AudioFormat getFormat() {
            return this.targetFormat;
        }
        
        @Override
        public long getFrameLength() {
            return -1L;
        }
        
        @Override
        public void mark(int n) {
            this.ais.mark((int)((float)n * this.pitch[0]));
            this.mark_ibuffer_index = this.ibuffer_index;
            this.mark_ibuffer_len = this.ibuffer_len;
            if (this.mark_ibuffer == null) {
                this.mark_ibuffer = new float[this.ibuffer.length][this.ibuffer[0].length];
            }
            for (int i = 0; i < this.ibuffer.length; ++i) {
                float[] array = this.ibuffer[i];
                float[] array2 = this.mark_ibuffer[i];
                for (int j = 0; j < array2.length; ++j) {
                    array2[j] = array[j];
                }
            }
        }
        
        @Override
        public boolean markSupported() {
            return this.ais.markSupported();
        }
        
        private void readNextBuffer() throws IOException {
            if (this.ibuffer_len == -1) {
                return;
            }
            for (int i = 0; i < this.nrofchannels; ++i) {
                float[] array = this.ibuffer[i];
                for (int n = this.ibuffer_len + this.pad2, j = this.ibuffer_len, n2 = 0; j < n; ++j, ++n2) {
                    array[n2] = array[j];
                }
            }
            this.ibuffer_index -= (float)this.ibuffer_len;
            this.ibuffer_len = this.ais.read(this.ibuffer2);
            if (this.ibuffer_len >= 0) {
                while (this.ibuffer_len < this.ibuffer2.length) {
                    int read = this.ais.read(this.ibuffer2, this.ibuffer_len, this.ibuffer2.length - this.ibuffer_len);
                    if (read == -1) {
                        break;
                    }
                    this.ibuffer_len += read;
                }
                Arrays.fill(this.ibuffer2, this.ibuffer_len, this.ibuffer2.length, 0.0f);
                this.ibuffer_len /= this.nrofchannels;
            }
            else {
                Arrays.fill(this.ibuffer2, 0, this.ibuffer2.length, 0.0f);
            }
            int length = this.ibuffer2.length;
            for (int k = 0; k < this.nrofchannels; ++k) {
                float[] array2 = this.ibuffer[k];
                for (int l = k, pad2 = this.pad2; l < length; l += this.nrofchannels, ++pad2) {
                    array2[pad2] = this.ibuffer2[l];
                }
            }
            return;
        }
        
        @Override
        public int read(float[] array, int n, int n2) throws IOException {
            if (this.cbuffer == null || this.cbuffer[0].length < n2 / this.nrofchannels) {
                this.cbuffer = new float[this.nrofchannels][n2 / this.nrofchannels];
            }
            if (this.ibuffer_len == -1) {
                return -1;
            }
            if (n2 < 0) {
                return 0;
            }
            int n3 = n + n2;
            int i = n2 / this.nrofchannels;
            int n4 = 0;
            int n5 = this.ibuffer_len;
            while (i > 0) {
                if (this.ibuffer_len >= 0) {
                    if (this.ibuffer_index >= (float)(this.ibuffer_len + this.pad)) {
                        this.readNextBuffer();
                    }
                    n5 = this.ibuffer_len + this.pad;
                }
                if (this.ibuffer_len < 0) {
                    n5 = this.pad2;
                    if (this.ibuffer_index >= (float)n5) {
                        break;
                    }
                }
                if (this.ibuffer_index < 0.0f) {
                    break;
                }
                for (int j = 0; j < this.nrofchannels; ++j) {
                    this.ix[0] = this.ibuffer_index;
                    this.ox[0] = n4;
                    this.resampler.interpolate(this.ibuffer[j], this.ix, (float)n5, this.pitch, 0.0f, this.cbuffer[j], this.ox, n2 / this.nrofchannels);
                }
                this.ibuffer_index = this.ix[0];
                n4 = this.ox[0];
                i -= n4 - n4;
            }
            for (int k = 0; k < this.nrofchannels; ++k) {
                int n6 = 0;
                float[] array2 = this.cbuffer[k];
                for (int l = k + n; l < n3; l += this.nrofchannels) {
                    array[l] = array2[n6++];
                }
            }
            return n2 - i * this.nrofchannels;
        }
        
        @Override
        public void reset() throws IOException {
            this.ais.reset();
            if (this.mark_ibuffer == null) {
                return;
            }
            this.ibuffer_index = this.mark_ibuffer_index;
            this.ibuffer_len = this.mark_ibuffer_len;
            for (int i = 0; i < this.ibuffer.length; ++i) {
                float[] array = this.mark_ibuffer[i];
                float[] array2 = this.ibuffer[i];
                for (int j = 0; j < array2.length; ++j) {
                    array2[j] = array[j];
                }
            }
            return;
        }
        
        @Override
        public long skip(long a) throws IOException {
            if (a < 0L) {
                return 0L;
            }
            if (this.skipbuffer == null) {
                this.skipbuffer = new float[1024 * this.targetFormat.getFrameSize()];
            }
            float[] skipbuffer = this.skipbuffer;
            while (a > 0L) {
                int read = this.read(skipbuffer, 0, (int)Math.min(a, (long)this.skipbuffer.length));
                if (read < 0) {
                    if (a == a) {
                        return (long)read;
                    }
                    break;
                }
                else {
                    a -= (long)read;
                }
            }
            return a - a;
        }
    }
}
